// イメージビュア 8x8版
// 公開された8x8ドット イメージデータを一覧・拡大表示します
// ※別途イメージデータファイル(Img8x8.int)が必要です
// STARTボタンを押すと終了します

// イメージデータについて
// http://www.picosoft.co.jp/ORANGE-light/SystemCharacters.pdf
// Copyright (c) 2019 Picosoft Co., Ltd.
// Made by Kaori Hanatsumi

//下記"18"を、IMG8x8c.intを保存したEEPROMスロット番号に書き換えてください
FSLOT = 18-1

' 色定数の定義
WHITE = rgb(255, 255, 255)
DARKGRAY = rgb(64, 64, 64)
ORANGE = rgb(255, 163, 0)
LBLUE = rgb(0, 140, 255)
DARKBLUE = rgb(0, 0, 64)
RED = rgb(255, 0, 0)

' ボタンの定義
BUTTON_UP = 5
BUTTON_DOWN = 8
BUTTON_LEFT = 7
BUTTON_RIGHT = 6
BUTTON_A = 11
BUTTON_B = 10
BUTTON_START = 9

' スプライトパターン定義
IMG_CSR = getmptr()
mdata 0xFE,0x7C,0x38,0x10

' ワークエリア定義
OFFSET = getmptr()
mptr OFFSET + 2
SBUF = getmptr()
mptr SBUF + 2
BUF = getmptr()
mptr BUF + 8
LED_WORK = getmptr()

HT16K33_ADDRESS = 0x70

i2ci 1,400
HT16K33_setup(1)
HT16K33_setBlink(1, 0)
HT16K33_setBrightness(1)
HT16K33_clear()

cls 2
for i=0 to 199
	img_get(i,0)
next

gprint 0,232,"Push START to Exit",ORANGE,0

' スプライト作成・表示
csr_r = sprnew(0,90-3,24*8-4,8,4,IMG_CSR,WHITE,0)
csr_g = sprnew(1,90-3,26*8-4,8,4,IMG_CSR,WHITE,0)
csr_b = sprnew(2,90-3,28*8-4,8,4,IMG_CSR,WHITE,0)
csr_sel = sprnew(3,90-3,24*8-4,8,4,IMG_CSR,RED,0)
sprdsp csr_r,1
sprdsp csr_g,1
sprdsp csr_b,1

' カラーバー描画
for i=0 to 63
	x=90+i
	rect x,24*8,1,7,rgb(i*4,0,0),1
	rect x,26*8,1,7,rgb(0,i*4,0),1
	rect x,28*8,1,7,rgb(0,0,i*4),1
next

' メインループ
i=0:img_zoom(i)
mode = 1
sel=0
while in(BUTTON_START)
	HT16K33_display()
	if trig_b() then change_mode()
	if mode then sel_img() else edit_color():pause 20
wend
end

' 動作モードを移行する
def change_mode() {
	mode = 1 - mode
// 色編集開始
	if mode=0 then csr_on() : return
// 色編集完了 : 色確定
	csr_off()
	pre_col = rgb(r*4, g*4, b*4)
	mpokew SBUF/2 ,pre_col
	index_write(i)
	img_zoom(i)
}
' イメージを選択する
def sel_img() {
	var pre_i
	pre_i = i
	i = i + trig_right() - trig_left() + trig_down()*10 - trig_up()*10
	i = i + ((i<0) - (i>199))*200
	if i <> pre_i then img_get(pre_i,0):img_zoom(i)
	if trig_a() then mpokew SBUF/2 ,pre_col:index_write(i):img_zoom(i)
}
' イメージデータを読んで表示
def img_get(n,flag) {
	var col1
	var col2
	index_read(n,SBUF)
	img_read(n,BUF)
	col = mpeekw(SBUF/2)
	col1 = col : col2 = DARKBLUE
	if flag then col1=DARKGRAY : col2=ORANGE
	gput8c(10 + n%10,n/10,BUF,col1,col2)
	return 0
}
' イメージデータを読んで拡大表示
def img_zoom(n) {
	var cx
	var cy
	var d
	var c
	if img_get(n,1) then rect 0,11,80,80,0,1 : return
	set_color_all(col)
	cy=7
	while cy>=0
		d=mpeek(BUF+cy)
		cx=7
		while cx>=0
			if d&1 then c=col else c=0
			rect cx*10+1,cy*10+11,8,8,c,1
			d=d>>1:cx=cx-1
		wend
		cy=cy-1
	wend
}

' 色コードをEEPROMから読み込む
def index_read(id,ptr) {
	var e
	mpokew OFFSET/2,FSLOT*2048 + 1632 + id*2
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,2,1)
}

' 色コードをEEPROMに書き込む
def index_write(id) {
	var e
	mpokew OFFSET/2,FSLOT*2048 + 1632 + id*2
	e = i2cw(1,0x50,OFFSET,4,1)
}

' イメージデータをEEPROMから読み込む
def img_read(id,ptr) {
	mpokew OFFSET/2,FSLOT*2048 + 32 + id*8
	e = i2cw(1,0x50,OFFSET,2,0)
	e = i2cr(1,0x50,ptr,8,1)
}

' 16ドット単位で8x8イメージデータを表示
def gput8c(x,y,adr,col1,col2) {
	sprnew(5,x*16,y*12,8,8,adr,col1,col2)
	sprdsp 5,1
}
' 色を編集する
def edit_color() {
	change_sel(trig_down() - trig_up())
	change_col(!in(BUTTON_RIGHT)*(s<63) - !in(BUTTON_LEFT)*(s>0))
}

' 色の変更
def change_col(d) {
	if d=0 then return
	s=s+d
	sprmov csr_sel,87+s,24*8-4 + sel*16
	set_color(s)
	rect 65,24*8,20,40,rgb(r*4,g*4,b*4),1
}

' RGBバー間のカーソル移動
def change_sel(d) {
	if d=0 then return
	csr_off()
	sel=(sel+d+3)%3
	csr_on()
}

' 色選択カーソルを非表示にする
def csr_off() {
	sprdsp csr_sel,0
	sprmov sel,87+s,24*8-4 + sel*16
	sprdsp sel,1
}

' 色選択カーソルを表示する
def csr_on() {
	s=get_color()
	sprdsp sel,0
	sprmov csr_sel,87+s,24*8-4 + sel*16
	sprdsp csr_sel,1
}

' 選択している色の輝度を取得
def get_color() {
	if sel=0 then return r
	if sel=1 then return g
	if sel=2 then return b
}

' 選択している色の輝度を変更
def set_color(n) {
	if sel=0 then r=n
	if sel=1 then g=n
	if sel=2 then b=n
}

' 色コードからrgbをまとめて設定
def set_color_all(c) {
	r = (c >> 10) & 0x3e
	g = (c >> 5) & 0x3f
	b = (c << 1) & 0x3e
	sprmov csr_r,87+r,24*8-4
	sprmov csr_g,87+g,26*8-4
	sprmov csr_b,87+b,28*8-4
	rect 65,24*8,20,40,c,1
}

' ボタン状態が変化＆押された時だけ1になる関数
def trig_down() {
	var ret
	ret = pre_d
	pre_d = in(BUTTON_DOWN)
	return ret & !pre_d
}
def trig_up() {
	var ret
	ret = pre_u
	pre_u = in(BUTTON_UP)
	return ret & !pre_u
}
def trig_left() {
	var ret
	ret = pre_l
	pre_l = in(BUTTON_LEFT)
	return ret & !pre_l
}
def trig_right() {
	var ret
	ret = pre_r
	pre_r = in(BUTTON_RIGHT)
	return ret & !pre_r
}
def trig_a() {
	var ret
	ret = pre_a
	pre_a = in(BUTTON_A)
	return ret & !pre_a
}
def trig_b() {
	var ret
	ret = pre_b
	pre_b = in(BUTTON_B)
	return ret & !pre_b
}

def HT16K33_setup(sw) {
	mpoke LED_WORK, 0x20 | sw
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 1, 1)
	pause 1000
}

def HT16K33_setBlink(sw, f) {
	mpoke LED_WORK, 0x80 | (f << 1) | sw
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 1, 1)
}

def HT16K33_setBrightness(level) {
	mpoke LED_WORK, 0xe0 | level
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 1, 1)
}

def HT16K33_clear() {
	mptr LED_WORK
	mdata 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 17, 1)
}

def HT16K33_display() {
	var i
	var d
	var s
	mptr LED_WORK
	mdata 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	mdata 0, 0, 0, 0, 0, 0, 0, 0
	for i = 0 to 7
		s = mpeek(BUF + i)
		d = 0
		if s & 0x80 then d = d | 0x80
		if s & 0x40 then d = d | 0x01
		if s & 0x20 then d = d | 0x02
		if s & 0x10 then d = d | 0x04
		if s & 0x08 then d = d | 0x08
		if s & 0x04 then d = d | 0x10
		if s & 0x02 then d = d | 0x20
		if s & 0x01 then d = d | 0x40
		mpoke LED_WORK + 1 + i * 2, d
	next
	i2cw(1, HT16K33_ADDRESS, LED_WORK, 17, 1)
}
